@@ -2,7 +2,8 @@
 
 This module is an argument serialization and deserialization library that:
 
-    - handles two-way conversions: a typed immutable argument object (A `NamedTuple` or `dataclass` instance) <==> a command-line argv
+    - handles two-way conversions: a typed, preferably immutable argument container object
+    (a `NamedTuple` or `dataclass` instance) <==> a command-line argv
     - enables IDE type hints and code auto-completion by using `NamedTuple` or `dataclass`
     - promotes type-safety of cli-ready arguments
 
@@ -41,7 +42,7 @@ The following is a simple usage example::
     arg.__to_argv__()
 
 
-The module contains the following public classes:
+The module contains the following public classes/functions:
 
 - `arg_suite` -- The main entry point for Smart Argument Suite. As the
     example above shows, this decorator will attach an `ArgSuite` instance to
@@ -55,25 +56,28 @@ The module contains the following public classes:
 
 All other classes and methods in this module are considered implementation details."""
 
-__version__ = '0.4.3'
-__all__ = (
-    'arg_suite',
-    'custom_arg_suite',
-    'LateInit',
-    'SmartArgError',
-    'TypeHandler',
-    'PrimitiveHandlerAddon',
-)
-
+import pkg_resources
 import logging
 import os
 import sys
 import warnings
 from argparse import Action, ArgumentParser
+from collections import abc
 from types import SimpleNamespace as KwargsType
 from typing import Any, Callable, Dict, Generic, Iterable, List, NamedTuple, Optional, Sequence, Tuple, Type, TypeVar, Union
 from enum import EnumMeta, Enum
 
+_base_version = '0.4.*'  # star version should be auto-resolved to concrete number when run setup.py
+__version__ = pkg_resources.get_distribution(__name__).version
+__all__ = (
+    'arg_suite',
+    'custom_arg_suite',
+    'frozenlist',
+    'LateInit',
+    'SmartArgError',
+    'TypeHandler',
+    'PrimitiveHandlerAddon',
+)
 ArgType = TypeVar('ArgType', bound=NamedTuple)  # NamedTuple is not a real class bound, but setting `bound` to NamedTuple makes mypy happier
 NoneType = None.__class__
 FieldMeta = NamedTuple('FieldMeta', (('comment', str), ('default', Any), ('type', Type), ('optional', bool)))
@@ -107,6 +111,7 @@ else:
                       f"unsupported python version {sys.version_info} < 3.6. It might work with 'pip install typing_inspect'.")
         raise
 
+frozenlist = tuple  # tuple to simulate an immutable list. [https://www.python.org/dev/peps/pep-0603/#rationale]
 _black_hole = lambda *args, **kwargs: None
 _mro_all = lambda arg_class, get_dict: {k: v for b in (*arg_class.__mro__[-1:0:-1], arg_class) if b.__module__ != 'builtins' for k, v in get_dict(b).items()}
 # note: Fields without type annotation won't be regarded as a property/entry _fields.
@@ -192,16 +197,11 @@ class TypeHandler:
         help_builder = ['(', 'Optional[' if field_meta.optional else '', self._type_to_str(arg_type), ']' if field_meta.optional else '']
         # Get default if specified and set required if no default
         kwargs.required = parent_required and field_meta.default is MISSING
-        if kwargs.required:
-            help_builder.append('; required')
-        else:
-            # Only add default to the help message for informational purpose. The default is set when creating the argument class instance.
-            help_builder.append('; default: ')
-            help_builder.append(str(field_meta.default))
-
-        help_builder.append(') ')
-        # Add from source code comment
-        help_builder.append(field_meta.comment)
+        message_required = 'required' if kwargs.required else \
+            'to be post-processed' if field_meta.default is LateInit else \
+            'required if its container is being specified or marked' if field_meta.default is MISSING else \
+            f'default: {field_meta.default}'  # informational only. The default is set when creating the argument container instance.
+        help_builder.extend(('; ', message_required, ') ', field_meta.comment))  # Add from source code comment
         kwargs.help = ''.join(help_builder)
 
     def _build_other(self, kwargs: KwargsType, arg_type: Type) -> None:
@@ -288,7 +288,7 @@ class CollectionHandler(TypeHandler):
 
     def handles(self, t: Type) -> bool:
         args = get_args(t)
-        return len(args) == 1 and get_origin(t) in (list, set) and _first_handles(self.primitive_addons, args[0], False)
+        return len(args) == 1 and get_origin(t) in (list, set, frozenset, abc.Sequence) and _first_handles(self.primitive_addons, args[0], False)
 
 
 class DictHandler(TypeHandler):
@@ -358,9 +358,13 @@ class _dataclasses:
         cls.__init__, cls.__original_init__ = init, cls.__init__
         cls.__setattr__ = lambda self, name, value: raise_if_frozen(self, '__setattr__', name, value)
         cls.__delattr__ = lambda self, name, : raise_if_frozen(self, '__delattr__', name)
+
+    @staticmethod
+    def field_default(arg_class, raw_arg_name):
+        f = arg_class.__dataclass_fields__[raw_arg_name]
+        return f.default_factory() if f.default_factory is not MISSING else f.default  # Assuming the default_factory has no side effects.
     proxy = lambda t: _dataclasses if is_dataclass(t) else None
     asdict = lambda args: asdict(args)
-    field_default = lambda arg_class, raw_arg_name: arg_class.__dataclass_fields__[raw_arg_name].default
     new_instance = lambda arg_class, _: arg_class.__original_new__(arg_class)
 
 
@@ -463,14 +467,11 @@ class ArgSuite(Generic[ArgType]):
                     self._gen_arguments_from_class(arg_type, f'{arg_name}.', required, arg_classes, sub_type_proxy)
                     arg_classes.pop()
                     kwargs = KwargsType(nargs='?',
-                                        required=False,
-                                        metavar='Help message only. Do NOT attempt to specify, or an exception will be raised.',
-                                        # capture the value of arg_name in the lambda since arg_name is not immutable
-                                        type=lambda _, arg=arg_name: _raise_if(f"Nested argument {arg!r} can not be directly specified"),
-                                        help=f"""This is a placeholder for the nested argument '{arg_name}'.
-                                             Its parent is {'' if parent_required else 'not'} required.
-                                             {"It's required" if default is MISSING else f"Not required with default: {default}"},
-                                             if the parent is being parsed.""")
+                                        required=required,
+                                        metavar='Does NOT accept any value',
+                                        # capture the value of arg_name in the lambda default argument since arg_name is not immutable
+                                        type=lambda _, arg=arg_name: _raise_if(f"Nested argument container marker {arg!r} does not accept any value."),
+                                        help=f"nested argument container marker{'' if default in (LateInit, MISSING) else f'; default: {default}'}")
                 else:
                     handler = _first_handles(self.handlers, arg_type)
                     field_meta = FieldMeta(comment=comments.get(raw_arg_name, ''), default=default, type=arg_type, optional=optional)
@@ -551,6 +552,8 @@ class ArgSuite(Generic[ArgType]):
                 arg_type = get_origin(t) or t
                 if arg_type == get_origin(Optional[Any]):
                     arg_type = get_args(t)
+                if arg_type is list and isinstance(attr, tuple) or arg_type is set and isinstance(attr, frozenset):  # allow for immutable replacements
+                    continue
                 try:
                     conforming = isinstance(attr, arg_type)  # e.g. list and List
                 except TypeError:  # best effort to check the instance type
